# python imports
import datetime

# django import
from django import template
from django.conf import settings
from django.core.cache import cache
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404
from django.template import Node, TemplateSyntaxError
from django.template.loader import render_to_string
from django.utils import translation
from django.utils.translation import ugettext_lazy as _

# contact_form imports
from contact_form.forms import ContactForm

# tagging imports
from tagging.managers import ModelTaggedItemManager

# feedparser imports
import feedparser

# lfc imports
import lfc.utils
from lfc.utils import registration
from lfc.models import BaseContent
from lfc.models import Page

register = template.Library()

@register.inclusion_tag('lfc/manage/templates.html', takes_context=True)
def templates(context):
    """Displays a selection box to select a available template for context
    on the fly.
    """
    lfc_context = context.get("lfc_context")
    request = context.get("request")

    if lfc_context is None:
        return {
            "display" : False,
        }

    templates = registration.get_templates(lfc_context)
    if templates and len(templates) > 1:
        display = True
        template = lfc_context.template or registration.get_default_template(lfc_context)
        if template:
            template_id = template.id
        else:
            template_id = None
    else:
        template_id = None
        display = False

    return {
        "display" : display,
        "templates" : templates,
        "obj_id" : lfc_context.id,
        "current_template" : template_id
    }

class LanguagesNode(Node):
    """
    """
    def render(self, context):
        lfc_context = context.get("lfc_context")
        request = context.get("request")

        languages = []
        for language in settings.LANGUAGES:
            if lfc_context is None:
                is_available = True
            elif lfc_context.has_language(request, language[0]):
                is_available = True
            else:
                is_available = False

            languages.append({
                "code" : language[0],
                "name" : language[1],
                "is_available" : is_available,
            })

        context["lfc_languages"] = languages
        return ''

def do_languages(parser, token):
    """Tag to put the contact form into context.
    """
    bits = token.contents.split()
    len_bits = len(bits)
    if len_bits != 1:
        raise TemplateSyntaxError(_('%s tag needs no argument') % bits[0])

    return LanguagesNode()

register.tag('lfc_languages', do_languages)

class ContactFormNode(Node):
    """Tag to put the contact form into context.
    """
    def render(self, context):
        request = context.get("request")
        if request.method == "POST":
            contact_form = ContactForm(data=request.POST, request=request)
        else:
            contact_form = ContactForm(request=request)

        context["form"] = contact_form
        return ''

def do_contact_form(parser, token):
    """Tag to put the contact form into context.
    """
    bits = token.contents.split()
    len_bits = len(bits)
    if len_bits != 1:
        raise TemplateSyntaxError(_('%s tag needs no argument') % bits[0])

    return ContactFormNode()

register.tag('contact_form', do_contact_form)

@register.inclusion_tag('lfc/tags/tabs.html', takes_context=True)
def tabs(context):
    """Returns the top level pages as tabs (aka horizontal menu bar).
    """
    request = context.get("request")
    language = context.get("LANGUAGE_CODE")

    lfc_context = context.get("lfc_context")

    if lfc_context:
        cache_key = "tabs-%s-%s" % (lfc_context.content_type, lfc_context.id)
    else:
        cache_key = "tabs-portal"

    pages = cache.get(cache_key)

    if pages is None:
        tl_objs = lfc.utils.get_content_objects(request,
            language__in=(language, "0"),
            parent = None,
            exclude_from_navigation=False,
        )

        if lfc_context is None:
            current_pages = []
        else:
            current_pages = [lfc_context]
            current_pages.extend(lfc_context.get_ancestors())

        tabs = []
        for obj in tl_objs:
            obj.current = obj in current_pages
            tabs.append(obj)

        cache.set(cache_key, pages)

    return {
        "language" : language,
        "tabs" : tabs,
        "portal" : lfc.utils.get_portal(),
    }

@register.inclusion_tag('lfc/tags/scrollable.html', takes_context=True)
def scrollable(context, tags=None, title=False, text=True, limit=5):
    """Displays objects with given tabs as scrollable
    """
    items = lfc.utils.get_content_object(request)

    if tags:
        items = ModelTaggedItemManager().with_all(tags, items)

    return {
        "items" : items,
        "title" : title,
        "text" : text,
    }

@register.inclusion_tag('lfc/tags/rss.html', takes_context=True)
def rss(context, url, limit=5):
    """An inclusion tag which displays an rss feed.
    """
    feed = feedparser.parse(url)

    try:
        name = feed["feed"]["link"].split("/")[-1]
    except (KeyError, IndexError, AttributeError):
        return {
            "entries" : [],
            "link" : "",
            "LANGUAGE_CODE" : "",
        }

    entries = []
    for entry in feed.entries[0:limit]:
        summary = entry.summary.replace("%s: " % name, "")

        entries.append({
            "title" : entry.title,
            "summary" : summary,
            "date" : datetime.datetime(*entry["updated_parsed"][0:6])
        })

    return {
        "entries" : entries,
        "LANGUAGE_CODE" : context.get("LANGUAGE_CODE"),
        "link" : feed["feed"]["link"],
    }

@register.inclusion_tag('lfc/tags/navigation.html', takes_context=True)
def navigation(context, start_level=1, expand_level=0):
    """Tag to render a navigation tree. This is also reused by the navigation
    portlet.
    """
    request = context.get("request")
    obj = request.META.get("lfc_context")

    language = translation.get_language()
    
    temp = lfc.utils.get_content_objects(request,
        parent = None,
        language__in = (language, "0"),
        exclude_from_navigation=False)

    # Add portal's standard to current_objs
    if obj is None:
        current_objs = []
        standard = lfc.utils.get_portal().standard
        if standard:
            if language != standard.language:
                standard = standard.get_translation(request, language)
            if standard:
                current_objs.append(standard.get_content_object())
    else:
        current_objs = [obj]
        current_objs.extend(obj.get_ancestors())

    objs = []
    for obj in temp:
        if obj in current_objs:
            children = _navigation_children(request, current_objs, obj, start_level, expand_level)
            is_current = True
        elif expand_level >= 1 and start_level <= 1:
            children = _navigation_children(request, current_objs, obj, start_level, expand_level)
            is_current = False
        else:
            children = ""
            is_current = False

        objs.append({
            "id" : obj.id,
            "slug" : obj.slug,
            "title" : obj.title,
            "url" : obj.get_absolute_url(),
            "is_current" : is_current,
            "children" : children,
            "level" : 1
        })

    return {
        "objs" : objs,
        "show_level" : start_level==1
    }

def _navigation_children(request, current_objs, obj, start_level, expand_level, level=2):
    """Renders the children of given object as sub navigation tree.
    """
    obj = obj
    temp = obj.children.restricted(request).filter(
        exclude_from_navigation = False,
        language__in = (translation.get_language(), "0"),
    ).get_content_objects()
    
    objs = []
    for obj in temp:
        if obj in current_objs:
            children = _navigation_children(request, current_objs, obj, start_level, expand_level, level=level+1)
            is_current = True
        elif level <= expand_level and level >= start_level:
            children = _navigation_children(request, current_objs, obj, start_level, expand_level, level=level+1)
            is_current = False
        else:
            children = ""
            is_current = False

        objs.append({
            "id" : obj.id,
            "slug" : obj.slug,
            "title" : obj.title,
            "url" : obj.get_absolute_url(),
            "is_current" : is_current,
            "children" : children,
            "level" : level,
        })

    result = render_to_string("lfc/tags/navigation_children.html", {
        "objs" : objs,
        "show_level" : level >= start_level,
    })

    return result

@register.inclusion_tag('lfc/tags/breadcrumbs.html', takes_context=True)
def breadcrumbs(context):
    """Displays breadcrumbs.
    """
    obj = context.get("lfc_context")
    if obj is None:
        return {
            "obj" : None,
            "objs" : []
        }

    objs = []

    temp = obj
    while temp is not None:
        objs.insert(0, temp)
        temp = temp.parent

    return {
        "obj" : obj,
        "objs" : objs,
    }

@register.inclusion_tag('lfc/tags/page.html', takes_context=True)
def page(context, slug, part):
    """Displays the a part (title/text) of the page with passes slug.
    """
    request = context.get("request")
    page = lfc.utils.traverse_object(request, slug)

    if page:
        page = page.get_content_object()

    return { "page" : page, "part": part }

@register.inclusion_tag('lfc/tags/objects.html', takes_context=True)
def objects_by_slug(context, slug):
    """Display all sub objects of the object with given slug
    """
    request = context.get("request")
    try:
        obj = lfc.utils.traverse_object(request, slug)
    except Http404:
        return { "objs" : [] }

    objs = obj.children.restricted(request)

    return { "objs" : objs }

@register.inclusion_tag('lfc/tags/previous_next.html')
def previous_next_by_date(obj):
    """Displays previous/links by creation date for the given object.
    """
    try:
        previous = obj.get_previous_by_creation_date(parent=obj.parent)
    except ObjectDoesNotExist:
        previous = None

    try:
        next = obj.get_next_by_creation_date(parent=obj.parent)
    except ObjectDoesNotExist:
        next = None

    return {
        "previous" : previous,
        "next" : next,
    }

@register.inclusion_tag('lfc/tags/previous_next.html', takes_context=True)
def previous_next_by_position(context, obj):
    """Displays previous/links by position for the given object.
    """
    request = context.get("request")
    siblings = [o.get_content_object() for o in obj.parent.children.restricted(request)]
    current_position = siblings.index(obj)
    next_position = current_position + 1
    previous_position = current_position - 1

    if previous_position < 0:
        previous = None
    else:
        try:
            previous = siblings[previous_position]
        except IndexError:
            previous = None

    try:
        next = siblings[next_position]
    except IndexError:
        next = None

    return {
        "previous" : previous,
        "next" : next,
    }


# django imports
from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

# tagging imports
from tagging.forms import TagField

# lfc imports
from lfc.fields.autocomplete import AutoCompleteTagInput
from lfc.models import Page
from lfc.models import BaseContent
from lfc.models import Portal
from lfc.models import ContentTypeRegistration
from lfc.utils.registration import get_allowed_subtypes
from lfc.utils.registration import get_info

class ContentTypeRegistrationForm(forms.ModelForm):
    """
    """
    class Meta:
        model = ContentTypeRegistration
        exclude = ("type", "name")
        
class CommentsForm(forms.ModelForm):
    """
    """
    class Meta:
        model = Page
        fields = ("allow_comments", )

class CoreDataForm(forms.ModelForm):
    """Core date form for pages.
    """
    tags = TagField(widget=AutoCompleteTagInput(), required=False)

    class Meta:
        model = Page
        fields = ("title", "display_title", "slug", "description", "text", "tags")

class SEOForm(forms.ModelForm):
    """
    """
    class Meta:
        model = Page
        fields = ( "meta_keywords", "meta_description")

class MetaDataForm(forms.ModelForm):
    """
    """
    def __init__(self, *args, **kwargs):
        super(MetaDataForm, self).__init__(*args, **kwargs)

        instance = kwargs.get("instance").get_content_object()
        language = instance.language
        ctr = get_info(instance)

        if not getattr(settings, "LFC_MULTILANGUAGE"):
            del self.fields["canonical"]
            del self.fields["language"]

        # Templates - display only registered templates for this instance
        if len(ctr.templates.all()) < 2:
            del self.fields["template"]
        else:
            self.fields["template"].choices = [(template.id, _(template.name)) for template in ctr.templates.all()]

        # Canonical - display only pages with default language
        if settings.LFC_MULTILANGUAGE:
            if instance.is_canonical():
                del self.fields["canonical"]
            else:
                canonicals = [(p.id, p.title) for p in BaseContent.objects.filter(language=settings.LANGUAGE_CODE)]
                canonicals.insert(0, ("", "----------"))
                self.fields["canonical"].choices = canonicals

        # Parents - display only objects in the same or neutral language
        # exclude = [p.id for p in instance.children.all()]
        # exclude.append(instance.id)
        # 
        # parents = BaseContent.objects.exclude(pk__in=exclude)
        # if not language == "0":
        #     parents = parents.filter(language__in=(language, "0"))
        # 
        # parent_choices = []
        # for parent in parents:
        #     if ctr in get_allowed_subtypes(parent.get_content_object()):
        #         parent_choices.append((parent.id, parent.title))
        # 
        # if len(parent_choices) < 1:
        #     del self.fields["parent"]
        # else:
        #     parent_choices = sorted(parent_choices, lambda a, b: cmp(a[1], b[1]))
        #     parent_choices.insert(0, ("", "----------"))
        #     self.fields["parent"].choices = parent_choices

        # Standard - display only children of the current instance
        if not ctr.display_select_standard:
            del self.fields["standard"]
        else:
            children = instance.children.all()
            if len(children) == 0:
                del self.fields["standard"]
            else:
                if not language == "0":
                    children = instance.children.filter(language__in=(language, "0"))
                else:
                    children = instance.children.all()

                standards = [(p.id, p.title) for p in children]
                standards = sorted(standards, lambda a, b: cmp(a[1], b[1]))
                standards.insert(0, ("", "----------"))
                self.fields["standard"].choices = standards

        # Position
        if not ctr.display_position:
            del self.fields["position"]

    class Meta:
        model = Page
        fields = ("template", "standard", "active", "exclude_from_navigation", 
            "exclude_from_search", "language", "canonical", "publication_date", )

class PortalCoreForm(forms.ModelForm):
    """Form for portal core data.
    """
    def __init__(self, *args, **kwargs):
        super(PortalCoreForm, self).__init__(*args, **kwargs)

        children = BaseContent.objects.filter(parent=None)
        standards = [(p.id, p.title) for p in children]
        standards = sorted(standards, lambda a, b: cmp(a[1], b[1]))
        standards.insert(0, ("", "----------"))
        self.fields["standard"].choices = standards

    class Meta:
        model = Portal



# django imports
from django.test import TestCase

# lfc imports
from lfc.models import BaseContent
from lfc.models import Page
from lfc.tests.utils import create_request

class ManagerTestCase(TestCase):
    """
    """
    def setUp(self):
        self.p1 = Page.objects.create(title="Page 1", slug="page-1")
        self.p2 = Page.objects.create(title="Page 2", slug="page-2")
        self.p2.active = True
        self.p2.save()

    def test_get(self):
        """
        """
        obj = BaseContent.objects.get(pk=1)
        self.failUnless(isinstance(obj, BaseContent))

    def test_get_content_object(self):
        """
        """
        obj = BaseContent.objects.get(pk=1).get_content_object()
        self.failUnless(isinstance(obj, Page))

    def test_get_content_objects(self):
        """
        """
        obj = BaseContent.objects.filter(pk=1).get_content_objects()
        self.failUnless(isinstance(obj[0], Page))

    def test_restricted_content_objects(self):
        """
        """
        request = create_request()
        pages = Page.objects.restricted(request)
        self.assertEqual(len(pages), 2)

        for page in pages:
            self.failUnless(isinstance(page, Page))

        request.user.is_superuser = False
        pages = Page.objects.restricted(request)
        self.assertEqual(pages[0].title, self.p2.title)

    def test_restricted(self):
        """
        """
        request = create_request()
        pages = Page.objects.restricted(request)
        self.assertEqual(len(pages), 2)

        request.user.is_superuser = False
        pages = Page.objects.restricted(request)
        self.assertEqual(pages[0].title, self.p2.title)


# lfc imports
import lfc.utils
from django.conf import settings

def main(request):
    """context processor for LFC.
    """
    return {
        "PORTAL" : lfc.utils.get_portal(),
        "LFC_MULTILANGUAGE" : settings.LFC_MULTILANGUAGE,
    }



# django imports
from django.test import TestCase

# portlets imports
from portlets.models import PortletRegistration

# lfc imports
import lfc.utils.registration
from lfc.models import BaseContent
from lfc.models import Page
from lfc.models import Template
from lfc.models import ContentTypeRegistration

class RegistrationTestCase(TestCase):
    """Tests for registration related stuff.
    """
    def test_default_portlets(self):
        """Portlets which have to be registered at least.
        """
        portlets = ["Pages", "Random", "Text", "Navigation"]
        registered_portlets = [p.name for p in PortletRegistration.objects.all()]

        for portlet in portlets:
            self.failUnless(portlet in registered_portlets)

    def test_default_templates(self):
        """Templates which have to be registered.
        """
        templates = ["Plain", "Article", "Gallery", "Overview"]
        registered_templates = [t.name for t in Template.objects.all()]

        for template in templates:
            self.failUnless(template in registered_templates)

    def test_default_content_types(self):
        """ContentTypes which have to be registered by default.
        """
        content_types = ["Page"]
        registered_content_types = [ct.name for ct in ContentTypeRegistration.objects.all()]

        for content_type in content_types:
            self.failUnless(content_type in registered_content_types)

    def test_register_template(self):
        """Registering a new template.
        """
        lfc.utils.registration.register_template("New", "/path/to/nowhere.html", 23, 42)
        template = Template.objects.get(name="New")

    def test_register_content_type(self):
        """Registering a new content type.
        """
        class NewContent(BaseContent):
            pass

        lfc.utils.registration.register_content_type(
            NewContent, "New Content", sub_types=["Page"], templates=["Article"],
            default_template="Article")

        # by name
        info = lfc.utils.registration.get_info("newcontent")
        self.assertEqual(info.type, "newcontent")
        self.assertEqual(info.name, "New Content")
        self.assertEqual(info.display_select_standard, True)
        self.assertEqual(info.display_position, True)
        self.assertEqual(info.global_addable, True)
        self.assertEqual(info.subtypes.all()[0].name, "Page")
        self.assertEqual(info.templates.all()[0].name, "Article")
        self.assertEqual(info.default_template.name, "Article")

        # by object
        info = lfc.utils.registration.get_info(NewContent())
        self.assertEqual(info.type, "newcontent")
        self.assertEqual(info.type, "newcontent")
        self.assertEqual(info.name, "New Content")
        self.assertEqual(info.display_select_standard, True)
        self.assertEqual(info.display_position, True)
        self.assertEqual(info.global_addable, True)
        self.assertEqual(info.subtypes.all()[0].name, "Page")
        self.assertEqual(info.templates.all()[0].name, "Article")
        self.assertEqual(info.default_template.name, "Article")

    def test_unregister_content_type(self):
        """Unregistering a content type.
        """
        class NewContent(BaseContent):
            pass

        lfc.utils.registration.register_content_type(
            NewContent, "New Content", sub_types=["Page"], templates=["Article"],
            default_template="Article")

        info = lfc.utils.registration.get_info("newcontent")
        self.assertEqual(info.name, "New Content")

        lfc.utils.registration.unregister_content_type("New Content")
        info = lfc.utils.registration.get_info("newcontent")
        self.assertEqual(info, None)

    def test_register_sub_type(self):
        """
        """
        class NewContent(BaseContent):
            pass

        lfc.utils.registration.register_content_type(
            NewContent, "New Content", sub_types=["Page"], templates=["Article"],
            default_template="Article")

        lfc.utils.registration.register_sub_type(NewContent, "Page")

        new_info = lfc.utils.registration.get_info("newcontent")
        page_info = lfc.utils.registration.get_info("page")
        self.failUnless(new_info in page_info.subtypes.all())

        self.failUnless(new_info in lfc.utils.registration.get_allowed_subtypes(Page()))



from django.db import models

from django.db import models
from django.db.models.query import QuerySet

class BaseContentQuerySet(QuerySet):
    """Custom QuerySet for BaseContent.
    """
    def get_content_objects(self):
        """Returns a list of content objects. Should be used at the end of
        a QuerySet chain, e.g.::

            BaseContent.objects.filter(language="en").exclude(pk=1).get_content_objects()
        """
        result = []
        for obj in self.all():
            result.append(obj.get_content_object())

        return result

class BaseContentManager(models.Manager):
    """Custom manager for BaseContent.
    """
    def get_query_set(self):
        """Overwritten to return BaseContentQuerySet.
        """
        return BaseContentQuerySet(self.model)

    def restricted(self, request):
        """Returns a query set according to the permissions of the current
        user.
        """
        user = request.user
        if user and user.is_superuser:
            return self.get_query_set()
        else:
            return self.get_query_set().filter(active=True)


# django imports
from django.contrib.sitemaps import Sitemap

# lfc imports
from lfc.models import Page

class PageSitemap(Sitemap):
    """Google's XML sitemap for pages.
    """
    changefreq = "weekly"
    priority = 0.5

    def items(self):
        return Page.objects.filter(
            active=True,
            exclude_from_navigation=False,
        )

    def lastmod(self, obj):
        return obj.creation_date


# django imports
from django.test import TestCase

# lfc imports
from lfc.models import Page
from lfc.models import Portal
from lfc.tests.utils import create_request

class PortalTestCase(TestCase):
    """Some tests for the Portal class.
    """
    def setUp(self):
        """
        """
        self.p = Portal.objects.create()
        self.p.notification_emails = "john@doe.com, jane@doe.com"
        self.p1 = Page.objects.create(title="Page 1", slug="page-1")
        self.p11 = Page.objects.create(title="Page 1-1", slug="page-1-1", parent=self.p1)
        self.p111 = Page.objects.create(title="Page 1-1-1", slug="page-1-1-1", parent=self.p11)
        self.p12 = Page.objects.create(title="Page 1-2", slug="page-1-2", parent=self.p1)
        self.p12 = Page.objects.create(title="Page 2", slug="page-2")

    def test_get_children(self):
        """
        """
        request = create_request()
        children = self.p.get_children(request)
        self.assertEqual(len(children), 2)

        # The cildren have to be specific objects
        for child in children:
            self.failUnless(isinstance(child, Page))

        children = self.p.get_children(request, slug="page-2")
        self.assertEqual(len(children), 1)

        request.user.is_superuser = False

        # No page is active
        children = self.p.get_children(request)
        self.assertEqual(len(children), 0)

        children = self.p.get_children()
        self.assertEqual(len(children), 2)

        # Only page 1 is active
        self.p1.active = True
        self.p1.save()

        children = self.p.get_children(request)
        self.assertEqual(len(children), 1)

        children = self.p.get_children()
        self.assertEqual(len(children), 2)

        # Page 2 is not active
        children = self.p.get_children(slug="page-2")
        self.assertEqual(len(children), 1)

        children = self.p.get_children(request, slug="page-2")
        self.assertEqual(len(children), 0)

    def test_get_parent_for_portlets(self):
        """
        """
        self.assertEqual(self.p.get_parent_for_portlets(), None)

    def test_are_comments_allowed(self):
        """
        """
        self.assertEqual(self.p.are_comments_allowed(), False)

        self.p.allow_comments = True
        self.p.save()
        self.assertEqual(self.p.are_comments_allowed(), True)

    def test_get_template(self):
        """
        """
        self.assertEqual(self.p.get_template().name, "Article")

    def test_get_notification_emails(self):
        """
        """
        self.assertEqual(self.p.get_notification_emails(), ["john@doe.com", "jane@doe.com"])




# django imports
from django.test import TestCase

# lfc imports
import lfc.utils.registration
from lfc.models import BaseContent
from lfc.models import Page
from lfc.tests.utils import create_request

class UtilsTestCase(TestCase):
    """
    """
    def setUp(self):
        self.p1 = Page.objects.create(title="Page 1", slug="page-1")
        self.p2 = Page.objects.create(title="Page 2", slug="page-2")

    def test_get_content_object(self):
        """
        """
        # As superuser
        ct = lfc.utils.get_content_object(slug="page-1")
        self.assertEqual(ct.title, "Page 1")

        request = create_request()
        request.user.is_superuser = False

        # Without passed request
        ct = lfc.utils.get_content_object(slug="page-1")
        self.assertEqual(ct.title, "Page 1")

        # With passed request
        self.assertRaises(BaseContent.DoesNotExist, lfc.utils.get_content_object, request, slug="page-1")

    def test_get_content_objects(self):
        """
        """
        # As superuser
        ct = lfc.utils.get_content_objects(slug="page-1")
        self.assertEqual(ct[0].title, "Page 1")

        request = create_request()
        request.user.is_superuser = False

        # Without passed request
        ct = lfc.utils.get_content_objects(slug="page-1")
        self.assertEqual(ct[0].title, "Page 1")

        # With passed request
        ct = lfc.utils.get_content_objects(request, slug="page-1")
        self.assertEqual(ct, [])




# django imports
from django.conf.urls.defaults import *

# LFC Manage
urlpatterns = patterns('lfc.manage.views',
    url(r'^applications$', "applications", name="lfc_applications"),
    url(r'^application/(?P<name>\w+)$', "application", name="lfc_application"),
    url(r'^install-application/(?P<name>\w+)$', "install_application", name="lfc_install_application"),
    url(r'^uninstall-application/(?P<name>\w+)$', "uninstall_application", name="lfc_uninstall_application"),
    url(r'^reinstall-application/(?P<name>\w+)$', "reinstall_application", name="lfc_reinstall_application"),

    url(r'^copy/(?P<id>\d+)$', "lfc_copy", name="lfc_copy"),
    url(r'^cut/(?P<id>\d+)$', "cut", name="lfc_cut"),
    url(r'^paste/(?P<id>\d+)$', "paste", name="lfc_paste"),
    url(r'^paste$', "paste", name="lfc_paste"),

    url(r'^content-types$', "content_types", name="lfc_content_types"),
    url(r'^content-type/(?P<id>\d+)$', "content_type", name="lfc_content_type"),

    url(r'^filebrowser$', "filebrowser", name="lfc_filebrowser"),
    url(r'^fb-upload-image$', "fb_upload_image", name="lfc_fb_upload_image"),
    url(r'^fb-upload-file$', "fb_upload_file", name="lfc_fb_upload_file"),

    url(r'^add-object/(?P<id>\w+)$', "add_object", name="lfc_add_object"),
    url(r'^add-object$', "add_object", name="lfc_add_top_object"),
    url(r'^add-object/(?P<language>\w+)/(?P<id>\w+)$', "add_object", name="lfc_add_object"),

    url(r'^delete-object/(?P<id>\d+)$', "delete_object", name="lfc_delete_object"),
    url(r'^save-core-data/(?P<id>\d+)$', "core_data", name="lfc_save_core_data"),
    url(r'^save-meta-data/(?P<id>\d+)$', "meta_data", name="lfc_save_meta_data"),
    url(r'^save-seo/(?P<id>\d+)$', "manage_seo", name="lfc_save_seo"),

    url(r'^add-images/(?P<id>\d+)$', "add_images", name="lfc_add_images"),
    url(r'^page-images/(?P<id>\d+)$', "images", name="lfc_images"),
    url(r'^update-images/(?P<id>\d+)$', "update_images", name="lfc_update_images"),
    url(r'^update-images$', "update_images", name="lfc_update_images"),

    url(r'^add-files/(?P<id>\d+)$', "add_files", name="lfc_add_files"),
    url(r'^page-files/(?P<id>\d+)$', "files", name="lfc_files"),
    url(r'^update-files/(?P<id>\d+)$', "update_files", name="lfc_update_files"),

    url(r'^add-portal-images$', "add_portal_images", name="lfc_add_portal_images"),
    url(r'^portal-images$', "portal_images", name="lfc_portal_images"),

    url(r'^update-comments/(?P<id>\d+)$', "update_comments", name="lfc_update_comments"),

    url(r'^update-children/(?P<id>\d+)$', "update_children", name="lfc_update_children"),
    url(r'^update-portal-children$', "update_portal_children", name="lfc_update_portal_children"),

    url(r'^save-portal-core$', "portal_core", name="lfc_save_portal_core"),

    url(r'^add-portlet/(?P<object_type_id>\d+)/(?P<object_id>\d+)$', "add_portlet", name="lfc_add_portlet"),
    url(r'^update-portlets/(?P<object_type_id>\d+)/(?P<object_id>\d+)$', "update_portlets", name="lfc_update_portlets"),
    url(r'^delete-portlet/(?P<portletassignment_id>\d+)$', "delete_portlet", name="lfc_delete_portlet"),
    url(r'^edit-portlet/(?P<portletassignment_id>\d+)$', "edit_portlet", name="lfc_edit_portlet"),

    url(r'^save-translation', "save_translation", name="lfc_save_translation"),
    url(r'^(?P<id>\d+)/translate/(?P<language>\w{2})', "translate_object", name="lfc_translate_object"),

    url(r'^set-navigation-tree-language/(?P<language>\w{2})', "set_navigation_tree_language", name="lfc_set_navigation_tree_language"),
    url(r'^set-template$', "set_template", name="lfc_set_template"),

    url(r'^$', "portal", name="lfc_manage_portal"),
    url(r'^(?P<id>\d+)$', "manage_object", name="lfc_manage_object"),
)



import sys
import tempfile
import hotshot
import hotshot.stats
from cStringIO import StringIO

# django imports
from django.conf import settings
from django.http import Http404
from django.http import HttpResponseServerError
from django.shortcuts import get_object_or_404
from django.utils import translation

# lfc imports
from lfc.utils import traverse_object
from lfc.utils import get_portal
from lfc.utils import get_content_object

from lfc.models import BaseContent

class ProfileMiddleware(object):
    """
    Displays hotshot profiling for any view.
    http://yoursite.com/yourview/?prof

    Add the "prof" key to query string by appending ?prof (or &prof=)
    and you'll see the profiling results in your browser.
    It's set up to only be available in django's debug mode,
    but you really shouldn't add this middleware to any production configuration.
    * Only tested on Linux
    """
    def process_request(self, request):
        if request.GET.has_key('prof'):
            self.tmpfile = tempfile.NamedTemporaryFile()
            self.prof = hotshot.Profile(self.tmpfile.name)

    def process_view(self, request, callback, callback_args, callback_kwargs):
        if request.GET.has_key('prof'):
            return self.prof.runcall(callback, request, *callback_args, **callback_kwargs)

    def process_response(self, request, response):
        if request.GET.has_key('prof'):
            self.prof.close()

            out = StringIO()
            old_stdout = sys.stdout
            sys.stdout = out

            stats = hotshot.stats.load(self.tmpfile.name)
            # stats.strip_dirs()
            stats.sort_stats('cumulative', )
            # stats.sort_stats('time', )
            stats.print_stats()

            sys.stdout = old_stdout
            stats_str = out.getvalue()

            if response and response.content and stats_str:
                response.content = "<pre>" + stats_str + "</pre>"

        return response


class AJAXSimpleExceptionResponse:
    def process_exception(self, request, exception):
        if settings.DEBUG:
            if request.is_ajax():
                import sys, traceback
                (exc_type, exc_info, tb) = sys.exc_info()
                response = "%s\n" % exc_type.__name__
                response += "%s\n\n" % exc_info
                response += "TRACEBACK:\n"
                for tb in traceback.format_tb(tb):
                    response += "%s\n" % tb
                return HttpResponseServerError(response)

class LFCMiddleware:
    """Traverses the requested object, store this within request.META and sets
    the correct language.
    """
    def process_view(self, request, view_func, view_args, view_kwargs):
        language = view_kwargs.get("language")
        slug = view_kwargs.get("slug")

        if slug is None and language is None:
            return

        if language:
            # if settings.LFC_MULTILANGUAGE == False:
            #     raise Http404
            if language not in settings.LFC_LANGUAGE_IDS:
                raise Http404

            translation.activate(language)
        else:
            translation.activate(settings.LANGUAGE_CODE)

        if slug:
            obj = traverse_object(request, view_kwargs.get("slug"))
            request.META["lfc_context"] = obj
        else:
            portal = get_portal()
            if portal.standard:
                obj = get_content_object(portal=portal)
                if obj.language != language:
                    if obj.is_canonical():
                        t = obj.get_translation(request, language)
                        if t:
                            obj = t
                    else:
                        canonical = obj.get_canonical(request)
                        if canonical:
                            obj = canonical
                request.META["lfc_context"] = obj
            else:
                request.META["lfc_context"] = portal


# django imports
from django.conf import settings
from django.test import TestCase

# lfc imports
from lfc.manage.forms import CoreDataForm
from lfc.models import BaseContent
from lfc.models import Page
from lfc.models import Portal
from lfc.tests.utils import create_request

class PageTestCase(TestCase):
    """Tests for Page related stuff.
    """
    def setUp(self):
        """
        """
        self.p = Portal.objects.create()
        self.p1 = Page.objects.create(title="Page 1", slug="page-1")
        self.p11 = Page.objects.create(title="Page 1-1", slug="page-1-1", parent=self.p1)
        self.p111 = Page.objects.create(title="Page 1-1-1", slug="page-1-1-1", parent=self.p11)
        self.p12 = Page.objects.create(title="Page 1-2", slug="page-1-2", parent=self.p1)

    def test_page_defaults(self):
        """Tests the default values of a freshly added page.
        """
        self.assertEqual(self.p1.title, "Page 1")
        self.assertEqual(self.p1.slug, "page-1")
        self.assertEqual(self.p1.display_title, True)
        self.assertEqual(self.p1.position, 1)
        self.assertEqual(self.p1.language, "0")
        self.assertEqual(self.p1.canonical, None)
        self.assertEqual(self.p1.tags, "")
        self.assertEqual(self.p1.parent, None)
        self.assertEqual(self.p1.template, None)
        self.assertEqual(self.p1.standard, None)
        self.assertEqual(self.p1.active, False)
        self.assertEqual(self.p1.exclude_from_navigation, False)
        self.assertEqual(self.p1.exclude_from_search, False)
        self.assertEqual(self.p1.creator, None)
        self.assertEqual(len(self.p1.images.all()), 0)
        self.assertEqual(self.p1.allow_comments, 1)
        self.assertEqual(self.p1.searchable_text, "Page 1")

        self.assertEqual(self.p1.meta_keywords, "<tags>")
        self.assertEqual(self.p1.meta_description, "<description>")

        self.assertEqual(self.p1.content_type, "page")

    def test_get_absolute_url(self):
        """
        """
        self.p1.language = settings.LANGUAGE_CODE
        self.p1.save()

        url = self.p1.get_absolute_url()
        self.assertEqual(url, "/page-1")

        self.p11.language = settings.LANGUAGE_CODE
        self.p11.save()

        url = self.p11.get_absolute_url()
        self.assertEqual(url, "/page-1/page-1-1")

    def test_get_content_object(self):
        """
        """
        bc = BaseContent.objects.get(slug="page-1")
        ct = bc.get_content_object()

        self.assertEqual(ct, self.p1)

    def test_get_searchable_text(self):
        """
        """
        self.assertEqual(self.p1.get_searchable_text(), "Page 1")

    def test_get_form(self):
        """
        """
        form = self.p1.form()
        self.assertEqual(form.__class__, CoreDataForm)

    def test_get_ancestors(self):
        """
        """
        self.assertEqual(self.p1.get_ancestors(), [])
        self.assertEqual(self.p11.get_ancestors(), [self.p1])
        self.assertEqual(self.p111.get_ancestors(), [self.p11, self.p1])
        self.assertEqual(self.p111.get_ancestors_reverse(), [self.p1, self.p11])

    def test_get_image(self):
        """
        """
        self.assertEqual(self.p1.get_image(), None)

    def test_get_meta_keywords(self):
        """
        """
        self.assertEqual(self.p1.get_meta_keywords(), "")

        self.p1.tags = "Dog, Cat"
        self.assertEqual(self.p1.get_meta_keywords(), "Dog, Cat")

        self.p1.meta_keywords = "<title>"
        self.assertEqual(self.p1.get_meta_keywords(), "Page 1")

        self.p1.meta_keywords = "<description>"
        self.p1.description = "Description"
        self.assertEqual(self.p1.get_meta_keywords(), "Description")

        self.p1.meta_keywords = "<tags>, <title>, <description>"
        self.assertEqual(self.p1.get_meta_keywords(), "Dog, Cat, Page 1, Description")

    def test_get_meta_description(self):
        """
        """
        self.p1.description = "Description"
        self.assertEqual(self.p1.get_meta_description(), "Description")

        self.p1.tags = "Dog, Cat"
        self.p1.meta_description = "<tags>"
        self.assertEqual(self.p1.get_meta_description(), "Dog, Cat")

        self.p1.meta_description = "<title>"
        self.assertEqual(self.p1.get_meta_description(), "Page 1")

        self.p1.meta_description = "<tags>, <title>, <description>"
        self.assertEqual(self.p1.get_meta_description(), "Dog, Cat, Page 1, Description")

    def test_get_template(self):
        """
        """
        self.assertEqual(self.p1.get_template().name, "Article")

    def test_get_title(self):
        """
        """
        self.assertEqual(self.p1.get_title(), "Page 1")

        self.p1.display_title = False
        self.assertEqual(self.p1.get_title(), "")

    def test_is_canonical(self):
        """
        """
        self.assertEqual(self.p1.is_canonical(), True)

    def test_get_canonical(self):
        """
        """
        request = create_request()
        self.assertEqual(self.p1.get_canonical(request), self.p1)

    def test_is_translation(self):
        """
        """
        self.assertEqual(self.p1.is_translation(), False)

    def test_has_language(self):
        """
        """
        request = create_request()
        # Returns True if the object is neutral
        self.assertEqual(self.p1.has_language(request, "en-us"), True)
        self.assertEqual(self.p1.has_language(request, "de"), True)

        self.p1.language = "en-us"
        self.assertEqual(self.p1.has_language(request, "en-us"), True)
        self.assertEqual(self.p1.has_language(request, "de"), False)

    def test_get_translation(self):
        """
        """
        request = create_request()
        self.assertEqual(self.p1.get_translation(request, "en-us"), None)

    def test_are_comments_allowed(self):
        """
        """
        self.assertEqual(self.p1.are_comments_allowed(), False)

    def test_get_descendants(self):
        """
        """
        descendants = self.p1.get_descendants()
        self.assertEqual(len(descendants), 3)

        self.failUnless(self.p11 in descendants)
        self.failUnless(self.p111 in descendants)
        self.failUnless(self.p12 in descendants)

        descendants = self.p11.get_descendants()
        self.assertEqual(len(descendants), 1)
        self.failUnless(self.p111 in descendants)

    def test_get_children(self):
        """
        """
        request = create_request()
        children = self.p1.get_children(request)
        self.assertEqual(len(children), 2)

        # The cildren have to be specific objects
        for child in children:
            self.failUnless(isinstance(child, Page))

        children = self.p1.get_children(request, slug="page-1-1")
        self.assertEqual(len(children), 1)

        request.user.is_superuser = False

        # No page is active
        children = self.p1.get_children(request)
        self.assertEqual(len(children), 0)

        children = self.p1.get_children()
        self.assertEqual(len(children), 2)

        # Only page 1 is active
        self.p11.active = True
        self.p11.save()

        children = self.p1.get_children(request)
        self.assertEqual(len(children), 1)

        children = self.p1.get_children()
        self.assertEqual(len(children), 2)

        # Page 2 is not active
        children = self.p1.get_children(slug="page-1-2")
        self.assertEqual(len(children), 1)

        children = self.p1.get_children(request, slug="page-1-2")
        self.assertEqual(len(children), 0)


from django.conf import settings
from django.utils.translation import ugettext_lazy as _

ALLOW_COMMENTS_DEFAULT = 1
ALLOW_COMMENTS_TRUE    = 2
ALLOW_COMMENTS_FALSE   = 3
ALLOW_COMMENTS_CHOICES = (
    (ALLOW_COMMENTS_DEFAULT, _(u"Default")),
    (ALLOW_COMMENTS_TRUE,    _(u"Yes")),
    (ALLOW_COMMENTS_FALSE,   _(u"No")),
)

LANGUAGE_CHOICES = [("0", _(u"Neutral"),)]
LANGUAGE_CHOICES.extend(settings.LANGUAGES)

COPY = 1
CUT = 2


# django imports
from django.contrib.syndication.feeds import Feed
from django.core.exceptions import ObjectDoesNotExist

# tagging imports
from tagging.managers import ModelTaggedItemManager

# lfc imports
import lfc.utils
from lfc.models import BaseContent

class PageTagFeed(Feed):
    """Provides a feed for a given page and a given tag passed by the bits of an
    url.
    """
    def get_object(self, bits):
        if len(bits) not in (1, 2):
            raise ObjectDoesNotExist

        if len(bits) == 2:
            self.tag = bits[1]
        else:
            self.tag = None
        return BaseContent.objects.get(slug__exact=bits[0])

    def title(self, obj):
        portal = lfc.utils.get_portal()
        return "%s - %s" % (portal.title, obj.title)

    def link(self, obj):
        return obj.get_absolute_url()

    # def description(self, obj):
    #     return obj.description

    def items(self, obj):
        objs = obj.children.filter(active=True)
        if self.tag:
            objs = ModelTaggedItemManager().with_all(self.tag, objs)

        return objs


# python imports
import copy

# django imports
from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.contrib.comments.models import Comment
from django.db import IntegrityError
from django.db.models import Q
from django.http import HttpResponseRedirect
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils import simplejson
from django.utils import translation
from django.utils.translation import ugettext_lazy as _

# portlets imports
from portlets.utils import get_registered_portlets
from portlets.utils import get_slots
from portlets.models import PortletAssignment
from portlets.models import PortletBlocking
from portlets.models import PortletRegistration
from portlets.models import Slot

# lfc imports
import lfc.utils
from lfc.models import BaseContent
from lfc.models import ContentTypeRegistration
from lfc.manage.forms import CommentsForm
from lfc.manage.forms import MetaDataForm
from lfc.manage.forms import SEOForm
from lfc.manage.forms import PortalCoreForm
from lfc.manage.forms import ContentTypeRegistrationForm
from lfc.models import Application
from lfc.models import File
from lfc.models import Image
from lfc.settings import COPY, CUT
from lfc.utils import LazyEncoder
from lfc.utils import MessageHttpResponseRedirect
from lfc.utils import get_portal
from lfc.utils import import_module
from lfc.utils.registration import get_allowed_subtypes
from lfc.utils.registration import get_info

# Portal #####################################################################
##############################################################################

@login_required
def portal(request, template_name="lfc/manage/portal.html"):
    """Displays the main management screen of the portal with all tabs.
    """
    return render_to_response(template_name, RequestContext(request, {
        "core_data" : portal_core(request),
        "children" : portal_children(request),
        "portlets" : portlets_inline(request, get_portal()),
        "navigation" : navigation(request, None),
        "images" : portal_images(request, as_string=True),
        "menu" : portal_menu(request),
        "display_paste" : _display_paste(request)
    }))

@login_required
def portal_core(request, template_name="lfc/manage/portal_core.html"):
    """Displays the core data tab of the portal.
    """
    portal = get_portal()

    if request.method == "POST":
        form = PortalCoreForm(instance=portal, data=request.POST)
        if form.is_valid():
            form.save()

        html =  render_to_string(template_name, RequestContext(request, {
            "form" : form,
            "portal" : portal,
        }))

        html = (
            ("#data", html),
        )

        result = simplejson.dumps({
            "html" : html,
            "message" : _(u"Portal data has been saved.")},
            cls = LazyEncoder
        )
        result = HttpResponse(result)
    else:
        form = PortalCoreForm(instance=portal)

        result = render_to_string(template_name, RequestContext(request, {
            "form" : form,
            "portal" : portal,
        }))

    return result

@login_required
def portal_children(request, template_name="lfc/manage/portal_children.html"):
    """Displays the children tab of the portal.
    """
    language = request.session.get("nav-tree-lang", settings.LANGUAGE_CODE)
    children = lfc.utils.get_content_objects(parent = None, language__in=("0", language))
    return render_to_string(template_name, RequestContext(request, {
        "children" : children,
        "display_paste" : _display_paste(request),
    }))

@login_required
def portal_images(request, as_string=False, template_name="lfc/manage/portal_images.html"):
    """Displays the images tab of the portal management screen.
    """
    obj = get_portal()

    result = render_to_string(template_name, RequestContext(request, {
        "obj" : obj,
        "images" : obj.images.all(),
    }))

    if as_string:
        return result
    else:
        result = simplejson.dumps({
            "images" : result,
            "message" : _(u"Images has been added."),
        }, cls = LazyEncoder)

        return HttpResponse(result)

def portal_menu(request, template_name="lfc/manage/portal_menu.html"):
    """Displays the manage menu of the portal.
    """
    content_types = get_allowed_subtypes()
    return render_to_string(template_name, RequestContext(request, {
        "display_paste" : _display_paste(request),
        "display_content_menu" : len(get_allowed_subtypes()) > 1,
        "content_types" : get_allowed_subtypes(),
    }))

# actions
def update_portal_children(request):
    """Deletes/Updates the children of the portal with passed ids (via
    request body).
    """
    obj =  get_portal()
    action = request.POST.get("action")
    if action == "delete":
        message = _(u"Objects has been deleted.")
        for key in request.POST.keys():
            if key.startswith("delete-"):
                try:
                    id = key.split("-")[1]
                    child = lfc.utils.get_content_object(pk=id)
                    _remove_fks(child)
                    child.delete()
                except (IndexError, BaseContent.DoesNotExist):
                    pass
    elif action == "copy":
        message = _(u"Objects have been put to the clipboard.")
        ids = []
        for key in request.POST.keys():
            if key.startswith("delete-"):
                id = key.split("-")[1]
                ids.append(id)
            request.session["clipboard"] = ids
            request.session["clipboard_action"] = COPY
    elif action == "cut":
        message = _(u"Objects have been put to the clipboard.")
        ids = []
        for key in request.POST.keys():
            if key.startswith("delete-"):
                id = key.split("-")[1]
                ids.append(id)
            request.session["clipboard"] = ids
            request.session["clipboard_action"] = CUT
    elif action == "paste":
        message = _paste(request, None)
    else:
        message = _(u"Objects has been updated.")
        for key in request.POST.keys():
            if key.startswith("obj_id-"):
                id = key.split("-")[1]
                try:
                    child = BaseContent.objects.get(pk=id)
                except BaseContent.DoesNotExist:
                    pass
                else:
                    position = request.POST.get("position-%s" % id, "")
                    if position != "":
                        try:
                            child.position = position
                        except ValueError:
                            pass
                    child.active = request.POST.get("is_active-%s" % id, 0)
                    child.save()

    _update_positions(None)
    html = (
        ("#children", portal_children(request)),
        ("#navigation", navigation(request, None)),
        ("#menu", portal_menu(request)),
    )

    result = simplejson.dumps({
        "html" : html,
        "message" : message,
    }, cls = LazyEncoder)

    return HttpResponse(result)

def add_portal_images(request):
    """Adds images to the portal.
    """
    obj = get_portal()
    if request.method == "POST":
        for file_content in request.FILES.values():
            image = Image(content=obj, title=file_content.name)
            image.image.save(file_content.name, file_content, save=True)

    # Refresh positions
    for i, image in enumerate(obj.images.all()):
        image.position = i+1
        image.save()

    return HttpResponse(portal_images(request, id, as_string=True))

# Filebrowser ################################################################
##############################################################################

def filebrowser(request):
    """Displays files/images of the current object within the file browser
    popup of TinyMCE.
    """
    obj_id = request.GET.get("obj_id")

    try:
        obj = BaseContent.objects.get(pk=obj_id)
    except (BaseContent.DoesNotExist, ValueError):
        obj = None
        language = translation.get_language()
    else:
        language = obj.language

    if request.GET.get("type") == "image":
        portal = get_portal()

        if obj:
            images = obj.images.all()
        else:
            images = []
        return render_to_response("lfc/manage/filebrowser_images.html",
            RequestContext(request, {
            "obj_id" : obj_id,
            "images" : images,
            "portal_images" : portal.images.all(),
        }))
    else:
        if obj:
            files = obj.files.all()
        else:
            files = []
        base_contents = []
        for base_content in BaseContent.objects.filter(parent=None, language__in=("0", language)):
            base_contents.append({
                "title" : base_content.title,
                "url" : base_content.get_absolute_url(),
                "children" : _filebrowser_children(request, base_content),
            })

        return render_to_response("lfc/manage/filebrowser_files.html",
            RequestContext(request, {
            "obj_id" : obj_id,
            "files" : files,
            "objs" : base_contents,
        }))

def _filebrowser_children(request, obj):
    """
    """
    objs = []
    for obj in obj.children.restricted(request):
        objs.append({
            "title" : obj.title,
            "url" : obj.get_absolute_url(),
            "children" : _filebrowser_children(request, obj),
        })

    return render_to_string("lfc/manage/filebrowser_files_children.html", RequestContext(request, {
        "objs" : objs
    }))

def fb_upload_image(request):
    """Uploads an image within filebrowser.
    """
    obj_id = request.POST.get("obj_id")
    obj = BaseContent.objects.get(pk=obj_id)

    if request.method == "POST":
        for file_content in request.FILES.values():
            image = Image(content=obj, title=file_content.name)
            image.image.save(file_content.name, file_content, save=True)

    # Refresh positions
    for i, image in enumerate(obj.images.all()):
        image.position = i+1
        image.save()

    url = "%s?obj_id=%s&amp;type=image" % (reverse("lfc_filebrowser"), obj_id)
    return HttpResponseRedirect(url)

def fb_upload_file(request):
    """Uploads file within filebrowser.
    """
    obj_id = request.POST.get("obj_id")
    obj = BaseContent.objects.get(pk=obj_id)

    for file_content in request.FILES.values():
        file = File(content=obj, title=file_content.name)
        file.file.save(file_content.name, file_content, save=True)

    # Refresh positions
    for i, file in enumerate(obj.files.all()):
        file.position = i+1
        file.save()

    url = "%s?obj_id=%s" % (reverse("lfc_filebrowser"), obj_id)
    return HttpResponseRedirect(url)

# Portlets ###################################################################
##############################################################################

@login_required
def portlets_inline(request, obj, template_name="lfc/manage/portlets_inline.html"):
    """Displays the assigned portlets for given object.
    """
    portlet_types = get_registered_portlets()
    ct = ContentType.objects.get_for_model(obj)

    parent_for_portlets = obj.get_parent_for_portlets()
    if parent_for_portlets:
        parent_slots = get_slots(parent_for_portlets)
    else:
        parent_slots = None

    return render_to_string(template_name, RequestContext(request, {
        "slots" : get_slots(obj),
        "parent_slots" : parent_slots,
        "parent_for_portlets" : parent_for_portlets,
        "portlet_types" : PortletRegistration.objects.all(),
        "object" : obj,
        "object_type_id" : ct.id,
    }))

@login_required
def update_portlets(request, object_type_id, object_id):
    """Update portlets blocking.
    """
    # Get content type to which the portlet should be added
    object_ct = ContentType.objects.get(pk=object_type_id)
    obj = object_ct.get_object_for_this_type(pk=object_id)

    blocked_slots = request.POST.getlist("block_slot")

    for slot in Slot.objects.all():
        if str(slot.id) in blocked_slots:
            try:
                PortletBlocking.objects.create(
                    slot_id=slot.id, content_type_id=object_type_id, content_id=object_id)
            except IntegrityError:
                pass

        else:
            try:
                pb = PortletBlocking.objects.get(
                    slot=slot, content_type=object_type_id, content_id=object_id)
                pb.delete()
            except PortletBlocking.DoesNotExist:
                pass

        html = portlets_inline(request, obj)

    result = simplejson.dumps({
        "html" : html,
        "message" : _(u"Portlet has been updated.")},
        cls = LazyEncoder
    )
    return HttpResponse(result)

@login_required
def add_portlet(request, object_type_id, object_id, template_name="lfc/manage/portlet_add.html"):
    """Form and logic to add a new portlet to the object with given type and id.
    """
    # Get content type to which the portlet should be added
    object_ct = ContentType.objects.get(pk=object_type_id)
    obj = object_ct.get_object_for_this_type(pk=object_id)

    # Get the portlet type
    portlet_type = request.REQUEST.get("portlet_type", "")

    if request.method == "GET":
        try:
            portlet_ct = ContentType.objects.filter(model=portlet_type.lower())[0]
            mc = portlet_ct.model_class()
            form = mc().form(prefix="portlet")
            return render_to_response(template_name, RequestContext(request, {
                "form" : form,
                "object_id" : object_id,
                "object_type_id" : object_ct.id,
                "portlet_type" : portlet_type,
                "slots" : Slot.objects.all(),
            }))
        except ContentType.DoesNotExist:
            pass
    else:
        try:
            ct = ContentType.objects.filter(model=portlet_type.lower())[0]
            mc = ct.model_class()
            form = mc().form(prefix="portlet", data=request.POST)
            portlet = form.save()

            slot_id = request.POST.get("slot")
            position = request.POST.get("position")
            PortletAssignment.objects.create(
                slot_id=slot_id, content=obj, portlet=portlet, position=position)

            result = simplejson.dumps({
                "html" : portlets_inline(request, obj),
                "message" : _(u"Portlet has been added.")},
                cls = LazyEncoder
            )

            return HttpResponse(result)

        except ContentType.DoesNotExist:
            pass

@login_required
def delete_portlet(request, portletassignment_id):
    """Deletes a portlet for given portlet assignment.
    """
    try:
        pa = PortletAssignment.objects.get(pk=portletassignment_id)
    except PortletAssignment.DoesNotExist:
        pass
    else:
        pa.delete()
        url = request.META.get("HTTP_REFERER")
        msg = _(u"Portlet has been deleted.")
        return MessageHttpResponseRedirect(url, msg)

@login_required
def edit_portlet(request, portletassignment_id, template_name="lfc/manage/portlet_edit.html"):
    """Form and logic to edit the portlet of the given portlet assignment.
    """
    try:
        pa = PortletAssignment.objects.get(pk=portletassignment_id)
    except PortletAssignment.DoesNotExist:
        return ""

    if request.method == "GET":
        slots = []
        for slot in Slot.objects.all():
            slots.append({
                "id" : slot.id,
                "name" : slot.name,
                "selected" : slot.id == pa.slot.id,
            })

        form = pa.portlet.form(prefix="portlet")
        return render_to_response(template_name, RequestContext(request, {
            "form" : form,
            "portletassigment_id" : pa.id,
            "slots" : slots,
            "position" : pa.position,
        }))
    else:
        form = pa.portlet.form(prefix="portlet", data=request.POST)
        portlet = form.save()

        # Save the rest
        pa.slot_id = request.POST.get("slot")
        pa.position = request.POST.get("position")
        pa.save()

        html = portlets_inline(request, pa.content)

        result = simplejson.dumps({
            "html" : html,
            "message" : _(u"Portlet has been saved.")},
            cls = LazyEncoder
        )
        return HttpResponse(result)

# Objects ####################################################################
##############################################################################

@login_required
def add_object(request, language=None, id=None):
    """Adds a new content object to the object with the passed id. if the passed
    id is None th content object is added to the portal.
    """
    type = request.REQUEST.get("type", "page")
    ct = ContentType.objects.filter(model=type)[0]
    mc = ct.model_class()
    form = mc().form

    try:
        parent_object = lfc.utils.get_content_object(pk=id)
    except (BaseContent.DoesNotExist, ValueError):
        parent_object = None

    if language is None:
        language = settings.LANGUAGE_CODE

    if request.method == "POST":
        form = form(data=request.POST, initial={"creator" : request.user})
        if request.POST.get("save"):
            if form.is_valid():
                # figure out language for new object
                if parent_object:
                    language = parent_object.language
                else:
                    language = request.session.get("nav-tree-lang", settings.LANGUAGE_CODE)

                new_object = form.save()
                new_object.parent = parent_object
                new_object.creator = request.user
                new_object.language = language
                new_object.position = 1000
                new_object.save()

                _update_positions(new_object, True)
                url = reverse("lfc_manage_object", kwargs={"id": new_object.id})
                msg = _(u"Page has been added.")
                return MessageHttpResponseRedirect(url, msg)

        else:
            referer = request.POST.get("referer")
            return HttpResponseRedirect(referer)
    else:
        if parent_object is not None:
            form = form(initial={"parent" : parent_object.id})
        else:
            form = form()

    if parent_object:
        parent_object = parent_object.get_content_object()

    return render_to_response("lfc/manage/object_add.html", RequestContext(request, {
        "type" : type,
        "name" : get_info(type).name,
        "form" : form,
        "language" : language,
        "id" : id,
        "referer" : request.POST.get("referer", request.META.get("HTTP_REFERER")),
        "navigation" : navigation(request, parent_object)
    }))

@login_required
def delete_object(request, id):
    """Deletes the content object with passed id.
    """
    try:
        obj = lfc.utils.get_content_object(pk = id)
    except BaseContent.DoesNotExist:
        pass
    else:
        parent = obj.parent
        _remove_fks(obj)
        obj.delete()

    if parent:
        url = reverse("lfc_manage_object", kwargs={"id": parent.id})
    else:
        url = reverse("lfc_manage_portal")

    msg = _(u"Page has been deleted.")
    return MessageHttpResponseRedirect(url, msg)

@login_required
def manage_object(request, id, template_name="lfc/manage/object.html"):
    """Displays the main management screen with all tabs of the content object
    with passed id.
    """
    try:
        obj = lfc.utils.get_content_object(pk=id)
    except BaseContent.DoesNotExist:
        url = reverse("lfc_manage_portal")
        return HttpResponseRedirect(url)

    return render_to_response(template_name, RequestContext(request, {
        "navigation" : navigation(request, obj),
        "menu" : object_menu(request, obj),
        "core_data" : core_data(request, id),
        "meta_data" : meta_data(request, id),
        "seo_data" : manage_seo(request, id),
        "images" : images(request, id, as_string=True),
        "files" : files(request, id),
        "comments" : comments(request, obj),
        "portlets" : portlets_inline(request, obj),
        "children" : children(request, obj),
        "content_type_name" : get_info(obj).name,
        "display_paste" : _display_paste(request),
    }))

def object_menu(request, obj, template_name="lfc/manage/object_menu.html"):
    """Displays the manage menu for the passed object.
    """
    if obj.is_canonical():
        canonical = obj
    else:
        canonical = obj.canonical

    languages = []
    for language in settings.LANGUAGES:
        if language[0] != settings.LANGUAGE_CODE:
            languages.append({
                "code" : language[0],
                "name" : language[1],
            })

    if canonical:
        translations = canonical.translations.all()
    else:
        translations = None

    content_types = get_allowed_subtypes(obj)
    
    return render_to_string(template_name, RequestContext(request, {
        "content_types" : content_types,
        "display_content_menu" : len(content_types) > 1,
        "translations" : translations,
        "languages" : languages,
        "canonical" : canonical,
        "obj" : obj,
        "display_paste" : _display_paste(request),
    }))

@login_required
def core_data(request, id, template_name="lfc/manage/object_data.html"):
    """Displays/Updates the core data tab of the content object with passed id.
    """
    obj = lfc.utils.get_content_object(pk=id)
    obj_ct = ContentType.objects.filter(model=obj.content_type)[0]

    Form = obj.form

    if request.method == "POST":
        form = Form(instance=obj, data=request.POST)
        if form.is_valid():
            form.save()

        html =  render_to_string(template_name, RequestContext(request, {
            "form" : form,
            "obj" : obj,
        }))

        html = (
            ("#data", html),
            ("#navigation", navigation(request, obj)),
        )

        result = simplejson.dumps({
            "html" : html,
            "message" : _(u"Data has been saved."),
        }, cls = LazyEncoder)
        result = HttpResponse(result)
    else:
        form = Form(instance=obj)

        result = render_to_string(template_name, RequestContext(request, {
            "form" : form,
            "obj" : obj,
        }))

    return result

@login_required
def meta_data(request, id, template_name="lfc/manage/object_meta_data.html"):
    """Displays/Updates the meta tab of the content object with passed id.
    """
    obj = lfc.utils.get_content_object(pk=id)

    if request.method == "POST":

        form = MetaDataForm(instance=obj, data=request.POST)

        if form.is_valid():
            form.save()
            form = MetaDataForm(instance=_update_positions(obj, True))

        html =  render_to_string(template_name, RequestContext(request, {
            "form" : form,
            "obj" : obj,
        }))

        html = (
            ("#meta_data", html),
            ("#navigation", navigation(request, obj)),
        )
        result = simplejson.dumps({
            "html" : html,
            "message" : _(u"Meta data has been saved."),
        }, cls = LazyEncoder)

        result = HttpResponse(result)

    else:
        form = MetaDataForm(instance=obj)

        result = render_to_string(template_name, RequestContext(request, {
            "form" : form,
            "obj" : obj,
        }))

    return result

@login_required
def manage_seo(request, id, template_name="lfc/manage/object_seo.html"):
    """Displays/Updates the SEO tab of the content object with passed id.
    """
    obj = lfc.utils.get_content_object(pk=id)

    if request.method == "POST":
        form = SEOForm(instance=obj, data=request.POST)
        if form.is_valid():
            form.save()

        html =  render_to_string(template_name, RequestContext(request, {
            "form" : form,
            "obj" : obj,
        }))

        html = (
            ("#seo", html),
        )
        result = simplejson.dumps({
            "html" : html,
            "message" : _(u"SEO has been saved."),
        }, cls = LazyEncoder)

        return HttpResponse(result)
    else:
        form = SEOForm(instance=obj)
        return render_to_string(template_name, RequestContext(request, {
            "form" : form,
            "obj" : obj,
        }))

@login_required
def children(request, obj, template_name="lfc/manage/object_children.html"):
    """Displays the children of the passed content object.
    """
    children = obj.children.all()
    return render_to_string(template_name, RequestContext(request, {
        "children" : children,
        "obj" : obj,
        "display_paste" : _display_paste(request),
    }))

@login_required
def update_children(request, id):
    """Deletes/Updates children for the content object with the passed id.The
    to updated children ids are passed within the request.
    """
    obj = lfc.utils.get_content_object(pk=id)

    action = request.POST.get("action")
    if action == "delete":
        message = _(u"Objects have been deleted.")
        for key in request.POST.keys():
            if key.startswith("delete-"):
                try:
                    id = key.split("-")[1]
                    child = lfc.utils.get_content_object(pk=id)
                    _remove_fks(child)
                    child.delete()
                except (IndexError, BaseContent.DoesNotExist):
                    pass
    elif action == "copy":
        message = _(u"Objects have been put to the clipboard.")
        ids = []
        for key in request.POST.keys():
            if key.startswith("delete-"):
                id = key.split("-")[1]
                ids.append(id)
            request.session["clipboard"] = ids
            request.session["clipboard_action"] = COPY
    elif action == "cut":
        message = _(u"Objects have been put to the clipboard.")
        ids = []
        for key in request.POST.keys():
            if key.startswith("delete-"):
                id = key.split("-")[1]
                ids.append(id)
            request.session["clipboard"] = ids
            request.session["clipboard_action"] = CUT
    elif action == "paste":
        message = _paste(request, id)
    else:
        message = _(u"Objects have been updated.")
        for key in request.POST.keys():
            if key.startswith("obj_id-"):
                id = key.split("-")[1]
                try:
                    child = BaseContent.objects.get(pk=id)
                except BaseContent.DoesNotExist:
                    pass
                else:
                    position = request.POST.get("position-%s" % id, "")
                    if position != "":
                        try:
                            child.position = position
                        except ValueError:
                            pass
                    child.active = request.POST.get("is_active-%s" % id, 0)
                    child.save()

    _update_positions(obj)
    html = (
        ("#navigation", navigation(request, obj.get_content_object())),
        ("#children", children(request, obj)),
        ("#menu", object_menu(request, obj)),
    )

    result = simplejson.dumps({
        "html" : html,
        "message" : message,
    }, cls = LazyEncoder)

    return HttpResponse(result)

# Comments ###################################################################
##############################################################################
@login_required
def comments(request, obj, template_name="lfc/manage/object_comments.html"):
    """Displays the comments tab of the passed object.
    """
    form = CommentsForm(instance=obj)
    comments = Comment.objects.filter(object_pk = obj.id)

    return render_to_string(template_name, RequestContext(request, {
        "obj" : obj,
        "comments" : comments,
        "form" : form,
    }))

@login_required
def update_comments(request, id):
    """Deletes/Updates comments from the object with passed id. The to updated
    comment ids are passed passed by request body.
    """
    obj = get_object_or_404(BaseContent, pk=id)

    action = request.POST.get("action")
    if action == "allow_comments":
        form = CommentsForm(instance=obj, data=request.POST)
        form.save()
        message = _(u"Allow comments state has been saved.")
    elif action == "delete":
        message = _(u"Comments has been deleted.")
        for key in request.POST.keys():
            if key.startswith("delete-"):
                try:
                    id = key.split("-")[1]
                    comment = Comment.objects.get(pk=id).delete()
                except (IndexError, Comment.DoesNotExist):
                    pass

    else:
        message = _(u"Comments has been updated.")
        for key in request.POST.keys():
            if key.startswith("comment_id-"):
                id = key.split("-")[1]
                try:
                    comment = Comment.objects.get(pk=id)
                except Comment.DoesNotExist:
                    pass
                else:
                    comment.is_public = request.POST.get("is_public-%s" % id, 0)
                    comment.save()

    html = (
        ("#comments", comments(request, obj)),
    )

    result = simplejson.dumps({
        "html" : html,
        "message" : message,
    }, cls = LazyEncoder)

    return HttpResponse(result)

# Files ######################################################################
##############################################################################
@login_required
def files(request, id, template_name="lfc/manage/object_files.html"):
    """Displays the files tab of the object with the passed id.
    """
    obj = lfc.utils.get_content_object(pk=id)
    return render_to_string(template_name, RequestContext(request, {
        "obj" : obj,
    }))

@login_required
def add_files(request, id):
    """Adds files to the object with the passed id. The to added files are passed
    within the request.
    """
    obj = lfc.utils.get_content_object(pk=id)
    if request.method == "POST":
        for file_content in request.FILES.values():
            file = File(content=obj, title=file_content.name)
            file.file.save(file_content.name, file_content, save=True)

    # Refresh positions
    for i, file in enumerate(obj.files.all()):
        file.position = i+1
        file.save()

    return HttpResponse(files(request, id))

@login_required
def update_files(request, id):
    """Saves/deletes files with for the object with the passed id. The to
    updated file ids are passed within the request.
    """
    obj = lfc.utils.get_content_object(pk=id)

    action = request.POST.get("action")
    if action == "delete":
        message = _(u"Files has been deleted.")
        for key in request.POST.keys():
            if key.startswith("delete-"):
                try:
                    temp_id = key.split("-")[1]
                    image = File.objects.get(pk=temp_id).delete()
                except (IndexError, File.DoesNotExist):
                    pass

    elif action == "update":
        message = _(u"Files has been updated.")
        for key, value in request.POST.items():
            if key.startswith("title-"):
                temp_id = key.split("-")[1]
                try:
                    file = File.objects.get(pk=temp_id)
                except File.DoesNotExist:
                    pass
                else:
                    file.title = value
                    file.save()

            elif key.startswith("position-"):
                try:
                    temp_id = key.split("-")[1]
                    file = File.objects.get(pk=temp_id)
                except (IndexError, File.DoesNotExist):
                    pass
                else:
                    file.position = value
                    file.save()

    # Refresh positions
    for i, file in enumerate(obj.files.all()):
        file.position = i+1
        file.save()

    result = simplejson.dumps({
        "files" : files(request, id),
        "message" : message,
    }, cls = LazyEncoder)

    return HttpResponse(result)

# Images #####################################################################

@login_required
def images(request, id, as_string=False, template_name="lfc/manage/object_images.html"):
    """Displays the images tab of a content object.
    """
    obj = lfc.utils.get_content_object(pk=id)

    result = render_to_string(template_name, RequestContext(request, {
        "obj" : obj,
    }))

    if as_string:
        return result
    else:
        result = simplejson.dumps({
            "images" : result,
            "message" : _(u"Images has been added."),
        }, cls = LazyEncoder)

        return HttpResponse(result)

def add_images(request, id):
    """Adds images to the object with the given id.

    The images are passed within the request body (request.FILES).
    """
    obj = lfc.utils.get_content_object(pk=id)

    if request.method == "POST":
        for file_content in request.FILES.values():
            image = Image(content=obj, title=file_content.name)
            image.image.save(file_content.name, file_content, save=True)

    # Refresh positions
    for i, image in enumerate(obj.images.all()):
        image.position = i+1
        image.save()

    return HttpResponse(images(request, id, as_string=True))

@login_required
def update_images(request, id=None):
    """Saves/deletes images for content object with passed id or the portal
    (if id is None).

    The to deleted images are passed within the request body.
    """
    if id is None:
        obj = get_portal()
    else:
        obj = lfc.utils.get_content_object(pk=id)
    action = request.POST.get("action")
    if action == "delete":
        message = _(u"Images has been deleted.")
        for key in request.POST.keys():
            if key.startswith("delete-"):
                try:
                    temp_id = key.split("-")[1]
                    image = Image.objects.get(pk=temp_id).delete()
                except (IndexError, Image.DoesNotExist):
                    pass

    elif action == "update":
        message = _(u"Images has been updated.")
        for key, value in request.POST.items():
            if key.startswith("title-"):
                temp_id = key.split("-")[1]
                try:
                    image = Image.objects.get(pk=temp_id)
                except Image.DoesNotExist:
                    pass
                else:
                    image.title = value
                    image.save()

            elif key.startswith("position-"):
                try:
                    temp_id = key.split("-")[1]
                    image = Image.objects.get(pk=temp_id)
                except (IndexError, Image.DoesNotExist):
                    pass
                else:
                    image.position = value
                    image.save()

    # Refresh positions
    for i, image in enumerate(obj.images.all()):
        image.position = i+1
        image.save()

    if id is None:
        images_inline = portal_images(request, as_string=True)
    else:
        images_inline = images(request, id, as_string=True)

    result = simplejson.dumps({
        "images" : images_inline,
        "message" : message,
    }, cls = LazyEncoder)

    return HttpResponse(result)

# Navigation tree ############################################################
##############################################################################

@login_required
def navigation(request, obj, start_level=1, template_name="lfc/manage/navigation.html"):
    """Displays the navigation tree of the management interfaces.
    """
    nav_tree_lang = request.session.get("nav-tree-lang", settings.LANGUAGE_CODE)

    if obj is None:
        current_objs = []
        is_portal = True
    else:
        current_objs = [obj]
        current_objs.extend(obj.get_ancestors())
        is_portal = False

    # Display all objs which are neutral or in default language
    q = Q(parent = None) & Q(language__in = ("0", nav_tree_lang))

    temp = BaseContent.objects.filter(q)

    objs = []
    for obj in temp:
        obj = obj.get_content_object()

        if obj in current_objs:
            children = _navigation_children(request, current_objs, obj, start_level)
            is_current = True
        else:
            children = ""
            is_current = False

        translations = []
        objs.append({
            "id" : obj.id,
            "title" : obj.title,
            "is_current" : is_current,
            "children" : children,
            "level" : 2,
            "translations" : obj.translations.all(),
        })

    languages = []
    for language in settings.LANGUAGES:
        if nav_tree_lang == language[0]:
            current_language = language[1]
        languages.append({
            "code" : language[0],
            "name" : language[1],
        })

    return render_to_string(template_name, RequestContext(request, {
        "objs" : objs,
        "show_level" : start_level==2,
        "level" : 2,
        "languages" : languages,
        "current_language": current_language,
        "is_portal" : is_portal,
    }))

def _navigation_children(request, current_objs, obj, start_level, level=3):
    """Renders the children of the given obj (recursively)
    """
    obj = obj.get_content_object()
    temp = obj.children.all()

    objs = []
    for obj in temp:
        obj = obj.get_content_object()
        if obj in current_objs:
            children = _navigation_children(request, current_objs, obj, start_level, level+1)
            is_current = True
        else:
            children = ""
            is_current = False

        objs.append({
            "id" : obj.id,
            "title" : obj.title,
            "is_current" : is_current,
            "children" : children,
            "level" : level,
        })

    result = render_to_string("lfc/manage/navigation_children.html", {
        "objs" : objs,
        "show_level" : level >= start_level,
        "level" : level,
    })

    return result

# actions
def set_navigation_tree_language(request, language):
    """Sets the language for the navigation tree.
    """
    request.session["nav-tree-lang"] = language
    return HttpResponseRedirect(request.META.get("HTTP_REFERER"))

# Translations ###############################################################
##############################################################################
@login_required
def translate_object(request, language, id=None, form_translation=None,
    form_canonical=None, template_name="lfc/manage/object_translate.html"):
    """Dislays the translation form for the object with passed id and language.
    """
    obj = get_object_or_404(BaseContent, pk=id)

    if obj.is_canonical():
        canonical = obj
        try:
            translation = obj.translations.filter(language=language)[0]
            translation_id = translation.id
        except IndexError:
            translation = None
            translation_id = ""
    else:
        translation = obj
        canonical = obj.canonical
        translation_id = translation.id

    if form_canonical is None:
        form_canonical = canonical.get_content_object().form(instance=canonical.get_content_object(), prefix="canonical")

    if translation:
        translation = translation.get_content_object()

    if form_translation is None:
        form_translation = canonical.get_content_object().form(instance=translation, prefix = "translation")

    return render_to_response(template_name, RequestContext(request, {
        "canonical" : canonical,
        "form_canonical" : form_canonical,
        "form_translation" : form_translation,
        "id" : id,
        "translation_language" : language,
        "translation_id" : translation_id,
    }))

@login_required
def save_translation(request):
    """Saves (adds or edits) a translation.
    """
    canonical_id = request.POST.get("canonical_id")

    if request.POST.get("cancel"):
        url = reverse("lfc_manage_object", kwargs={"id" : canonical_id})
        _(u"Translation has been canceled.")
        return MessageHttpResponseRedirect(url, msg)

    canonical = BaseContent.objects.get(pk=canonical_id)
    canonical = canonical.get_content_object()

    try:
        translation_id = request.POST.get("translation_id")
        translation = BaseContent.objects.get(pk=translation_id)
        translation = translation.get_content_object()
        translation_language = translation.language
        msg = _(u"Translation has been updated.")
    except (BaseContent.DoesNotExist, ValueError):
        translation = None
        translation_language = request.POST.get("translation_language")
        msg = _(u"Translation has been added.")

    # Get parent obj
    # 1. Take the translation of the parent if it available in requested language
    # 2. If not, take the parent of the canonical if it's in neutral language
    # 3. If not, don't take a parent at all
    parent = canonical.parent
    if parent == None:
        parent_translation = None
    else:
        try:
            parent_translation = parent.translations.get(language=translation_language)
        except (BaseContent.DoesNotExist):
            if parent.language == "0":
                parent_translation = parent
            else:
                parent_translation = None

    # Get standard obj
    try:
        standard = canonical.standard
        standard_translation = standard.translations.filter(language=translation_language)[0]
    except (AttributeError, IndexError):
        standard_translation = None

    form_canonical = canonical.form(
        prefix="canonical",
        instance = canonical,
        data=request.POST,
        files=request.FILES,
    )

    form_translation = canonical.form(
        prefix="translation",
        instance = translation,
        data=request.POST,
        files=request.FILES,
    )

    if form_canonical.is_valid() and form_translation.is_valid():
        translation = form_translation.save()
        translation.language = translation_language
        translation.canonical = canonical
        translation.parent = parent_translation
        translation.standard = standard_translation
        translation.template = canonical.template
        translation.content_type = canonical.content_type
        translation.creator = request.user
        translation.save()

        _update_positions(translation)

        url = reverse("lfc_manage_object", kwargs={"id" : translation.id})
        return MessageHttpResponseRedirect(url, msg)
    else:
        return translate_object(request, translation_language, canonical.id, form_translation, form_canonical)

# Template ###################################################################
##############################################################################
def set_template(request):
    """Sets the template of the current object
    """
    obj_id = request.POST.get("obj_id")
    template_id = request.POST.get("template_id")

    obj = BaseContent.objects.get(pk=obj_id)
    obj.template_id = template_id
    obj.save()

    return HttpResponseRedirect(obj.get_absolute_url())

# Cut/Copy and paste #########################################################
##############################################################################

def lfc_copy(request, id):
    """Puts the object with passed id into the clipboard.
    """
    request.session["clipboard"] = [id]
    request.session["clipboard_action"] = COPY

    url = reverse("lfc_manage_object", kwargs = { "id" : id })
    msg = _(u"The object has been put to the clipboard.")

    return MessageHttpResponseRedirect(url, msg)

def cut(request, id):
    """Puts the object within passed id into the clipboard and marks action
    as cut.
    """
    request.session["clipboard"] = [id]
    request.session["clipboard_action"] = CUT

    url = reverse("lfc_manage_object", kwargs = { "id" : id })
    msg = _(u"The object has been put to the clipboard.")

    return MessageHttpResponseRedirect(url, msg)

def paste(request, id=None):
    """Paste the object in the clipboard to object with given id.
    """
    if id:
        url = reverse("lfc_manage_object", kwargs = { "id" : id })
    else:
        url = reverse("lfc_manage_portal")

    msg = _paste(request, id)
    return MessageHttpResponseRedirect(url, msg)

def _paste(request, id):
    """
    """
    # Try to get the action
    action = request.session.get("clipboard_action", "")
    if action == "":
        _reset_clipboard(request)
        msg = _(u"An error has been occured. Please try again.")
        return msg

    # Get the target object
    if id:
        try:
            target = lfc.utils.get_content_object(pk=id)
        except BaseContent.DoesNotExist:
            msg = _(u"The target object has been deleted in the meanwhile.")
            return msg
    else:
        target = None

    # Get the source objs
    source_ids = request.session.get("clipboard", [])

    error_msg = ""
    for source_id in source_ids:
        try:
            source_obj = lfc.utils.get_content_object(pk=source_id)
        except BaseContent.DoesNotExist:
            error_msg = _(u"Some cut/copied objects has been deleted in the meanwhile.")
            continue

        # Copy only allowed sub types to target
        allowed_subtypes = get_allowed_subtypes(target)
        ctr_source = get_info(source_obj)

        if ctr_source not in allowed_subtypes:
            error_msg = _(u"Some objects are not allowed here.")
            continue

        # Don't copy to own descendants
        descendants = source_obj.get_descendants()
        if target in descendants or target == source_obj:
            error_msg = _(u"The objects can't be pasted in own descendants.")
            break

        if action == CUT:
            source_obj.parent_id = id
            source_obj.slug = _generate_slug(source_obj, target)
            source_obj.save()
            _reset_clipboard(request)
        else:
            # Here we go ...
            target_obj = copy.deepcopy(source_obj)
            target_obj.pk = None
            target_obj.id = None
            target_obj.parent_id = id
            target_obj.position = 1000

            target_obj.slug = _generate_slug(source_obj, target)
            target_obj.save()

            _copy_images(source_obj, target_obj)
            _copy_files(source_obj, target_obj)
            _copy_portlets(source_obj, target_obj)
            _copy_descendants(source_obj, target_obj)
            _copy_translations(source_obj, target_obj)

    _update_positions(target)

    if error_msg:
        msg = error_msg
    else:
        msg = _(u"The object has been pasted.")

    return msg

def _generate_slug(source_obj, parent):
    """Generates a unique slug for passed source_obj in passed parent
    """
    # Generate slug for pasted object
    new_slug = source_obj.slug
    try:
        BaseContent.objects.get(slug=new_slug, parent=parent, language=source_obj.language)
    except BaseContent.DoesNotExist:
        pass
    else:
        i = 1
        while 1:
            new_slug = source_obj.slug + "-%s" % i
            try:
                BaseContent.objects.get(slug=new_slug, parent=parent, language=source_obj.language)
            except BaseContent.DoesNotExist:
                break
            i += 1

    return new_slug

def _reset_clipboard(request):
    """Resets the clipboard.
    """
    if request.session.has_key("clipboard"):
        del request.session["clipboard"]
    if request.session.has_key("clipboard_action"):
        del request.session["clipboard_action"]

def _copy_descendants(source_obj, target_obj):
    """Copies all descendants of the passed object.
    """
    for child in source_obj.children.all().get_content_objects():
        new_child = copy.deepcopy(child)
        new_child.pk = None
        new_child.id = None
        new_child.parent = target_obj
        new_child.save()

        _copy_images(child, new_child)
        _copy_files(child, new_child)
        _copy_portlets(child, new_child)
        _copy_descendants(child, new_child)
        _copy_translations(child, new_child)

def _copy_images(source_obj, target_obj):
    """Copies all images from source_obj to target_obj.
    """
    for image in source_obj.images.all():
        new_image = Image(content=target_obj, title=image.title)
        new_image.image.save(image.image.name, image.image.file, save=True)
        new_image.save()

def _copy_files(source_obj, target_obj):
    """Copies all files from source_obj to target_obj.
    """
    for file in source_obj.files.all():
        new_file = File(content=target_obj, title=file.title)
        new_file.file.save(file.file.name, file.file.file, save=True)
        new_file.save()

def _copy_portlets(source_obj, target_obj):
    """Copies all portlets from source_obj to target_obj.
    """
    ct = ContentType.objects.get_for_model(source_obj)
    for pa in PortletAssignment.objects.filter(content_id=source_obj.id, content_type=ct):
        new_pa = copy.deepcopy(pa)
        new_pa.pk = None
        new_pa.id = None
        new_pa.content_id = target_obj.id
        new_pa.save()

def _copy_translations(source_obj, target_obj):
    """Copies all translations from source_obj to target_obj.
    """
    for translation in source_obj.translations.all().get_content_objects():
        new_translation = copy.deepcopy(translation)
        new_translation.pk = None
        new_translation.id = None
        new_translation.slug = _generate_slug(translation, translation.parent)
        new_translation.canonical = target_obj
        new_translation.save()

        _copy_images(translation, new_translation)
        _copy_files(translation, new_translation)
        _copy_portlets(translation, new_translation)
        _copy_descendants(translation, new_translation)

# Content types ##############################################################
##############################################################################
def content_types(request):
    """Redirects to the first content type.
    """
    ctr = ContentTypeRegistration.objects.filter()[0]
    url = reverse("lfc_content_type", kwargs={"id" : ctr.id })
    return HttpResponseRedirect(url)

def content_type(request, id, template_name="lfc/manage/content_types.html"):
    """ Displays the main screen of the content type management.
    """
    ctr = ContentTypeRegistration.objects.get(pk=id)

    if request.method == "POST":
        form = ContentTypeRegistrationForm(data = request.POST, instance=ctr)
        if form.is_valid():
            form.save()
    else:
        form = ContentTypeRegistrationForm(instance=ctr)

    return render_to_response(template_name, RequestContext(request, {
        "types" : ContentTypeRegistration.objects.all(),
        "ctr" : ctr,
        "form" : form,
    }))

# Applications ###############################################################
##############################################################################
def applications(request, template_name="lfc/manage/applications.html"):
    """Displays install/uninstall applications view.
    """
    applications = []
    for app_name in settings.INSTALLED_APPS:
        module = import_module(app_name)
        if hasattr(module, "install"):
            try:
                Application.objects.get(name=app_name)
            except Application.DoesNotExist:
                installed = False
            else:
                installed = True

            applications.append({
                "name" : app_name,
                "installed" : installed,
                "pretty_name" : getattr(module, "name", app_name),
                "description" : getattr(module, "description", None),
            })

    url = reverse("lfc_applications")
    return render_to_response(template_name, RequestContext(request, {
        "applications" : applications,
    }))

def install_application(request, name):
    """Installs LFC application with passed name.
    """
    import_module(name).install()
    try:
        Application.objects.create(name=name)
    except Application.DoesNotExist:
        pass

    url = reverse("lfc_applications")
    return HttpResponseRedirect(url)

def reinstall_application(request, name):
    """Reinstalls LFC application with passed name.
    """
    import_module(name).uninstall()
    import_module(name).install()
    try:
        Application.objects.create(name=name)
    except IntegrityError:
        pass

    url = reverse("lfc_applications")
    return HttpResponseRedirect(url)

def uninstall_application(request, name):
    """Uninstalls LFC application with passed name.
    """
    import_module(name).uninstall()

    try:
        application = Application.objects.get(name=name)
    except Application.DoesNotExist:
        pass
    else:
        application.delete()

    url = reverse("lfc_applications")
    return HttpResponseRedirect(url)

def application(request, name, template_name="lfc/manage/application.html"):
    """
    """
    url = reverse("lfc_application", kwargs={ "name" : name })
    return HttpResponseRedirect(url)

def _display_paste(request):
    """Returns true if the paste button should be displayed.
    """
    return request.session.has_key("clipboard")

def _remove_fks(obj):
    """Removes the objects from foreign key fields (in order to not delete
    these related objects)
    """
    parent = obj.parent
    if parent is None:
        parent = get_portal()

    if parent.standard and parent.standard.get_content_object() == obj:
        parent.standard = None
        parent.save()

    if obj.is_canonical():
        for t in obj.translations.all():
            t.canonical = None
            t.save()

def _update_positions(obj, take_parent=False):
    """Updates position of given object's children. If take_parent is True
    the children of the given object's parent are updated.
    """
    if take_parent == True:
        parent = obj.parent
    else:
        parent = obj

    for language in settings.LANGUAGES:
        if language[0] == settings.LANGUAGE_CODE:
            objs = BaseContent.objects.filter(parent=parent, language__in=("0", language[0]))
        else:
            objs = BaseContent.objects.filter(parent=parent, language = language[0])

        for i, p in enumerate(objs):
            p.position = (i+1)*10
            p.save()
            if obj and obj.id == p.id:
                obj = p

    return obj




# django imports
from django.conf import settings
from django.conf.urls.defaults import *
from django.views.generic.simple import direct_to_template

# tagging imports
from tagging.views import tagged_object_list

# lfc imports
from lfc.feeds import PageTagFeed
from lfc.models import Page
from lfc.sitemap import PageSitemap

# Tags
urlpatterns = patterns("lfc.views",
    url(r'tag/(?P<slug>[-\w]+)/(?P<tag>[^/]+)/$', "lfc_tagged_object_list", name="page_tag_detail"),
)

# Comments
urlpatterns += patterns("",
    (r'^comments/', include('django.contrib.comments.urls')),
)

# Login / Logout
urlpatterns += patterns('django.contrib.auth.views',
    url('^login/?$',  "login",  { "template_name" : "lfc/login.html" },  name='lfc_login'),
    url('^logout/?$', "logout", { "template_name" : "lfc/logged_out.html" }, name='lfc_logout'),
)

# Feeds
feeds = {
    "rss" : PageTagFeed,
}
urlpatterns += patterns('',
    url(r'(?P<url>rss.*)/$', 'django.contrib.syndication.views.feed', {'feed_dict': feeds}, name="feed"),
)

# Sitemaps
urlpatterns += patterns("django.contrib.sitemaps.views",
    url(r'^sitemap.xml', 'sitemap', {'sitemaps': {"pages": PageSitemap}})
)

# Robots
urlpatterns += patterns('django.views.generic.simple',
    (r'^robots.txt', 'direct_to_template', {'template': 'lfc/robots.txt'}),
)

# LFC
urlpatterns += patterns('lfc.views',
    url(r'^(?P<language>[-\w]{2})/search-results', "search_results", name="lfc_search"),

    url(r'^set-language/(?P<language>[-\w]{2})/$', 'set_language', name="lfc_set_language"),
    url(r'^set-language/(?P<language>[-\w]{2})/(?P<id>\d+)/$', 'set_language', name="lfc_set_language"),
    
    url(r'^file/(?P<id>[-\w]*)', "file", name="lfc_file"),

    url(r'^(?P<language>[-\w]{2})$', "base_view", name="lfc_base_view"),
    url(r'^(?P<language>[-\w]{2})/(?P<slug>.*)', "base_view", name="lfc_base_view"),
    url(r'^(?P<slug>.*)$', "base_view", name="lfc_base_view"),
)



# python imports
import datetime
import urllib
import sys


# django settings
from django.core.cache import cache
from django.http import Http404
from django.http import HttpResponseRedirect
from django.shortcuts import _get_queryset
from django.utils import simplejson
from django.utils.functional import Promise
from django.utils.encoding import force_unicode
from django.utils import translation

# lfc imports
import lfc.models

# TODO: Checkout Django's new message feature
class MessageHttpResponseRedirect(HttpResponseRedirect):
    """Specific HttpResponseRedirect to set a cookie with a message.
    """
    def __init__(self, redirect_to, message):
        HttpResponseRedirect.__init__(self, redirect_to)

        # We just keep the message two seconds.
        max_age = 2
        expires = datetime.datetime.strftime(
            datetime.datetime.utcnow() +
            datetime.timedelta(seconds=max_age), "%a, %d-%b-%Y %H:%M:%S GMT")

        self.set_cookie("message", lfc_quote(message), max_age=max_age, expires=expires)

class LazyEncoder(simplejson.JSONEncoder):
    """JSONEncoder which encodes django's lazy i18n strings.
    
    This is mainly used to return status messages along with content to ajax
    calls.
    """
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_unicode(obj)
        return obj

def get_content_object(request=None, *args, **kwargs):
    """Returns specific content object based on passed parameters.

    This method should be used if one wants the specific content object
    instead of the BaseContent object.

    You can consider this as the equivalent to Django's get method.
    """
    if request:
        obj = lfc.models.BaseContent.objects.restricted(request).get(*args, **kwargs)
    else:
        obj = lfc.models.BaseContent.objects.get(*args, **kwargs)

    return obj.get_content_object()

def get_content_objects(request=None, *args, **kwargs):
    """Returns specific content objects based on passed parameters.

    This method should be used if one wants the specific content objects
    instead of the BaseContent objects.

    You can consider this as the equivalent to Django's filter method.
    """

    if request:
        objs = lfc.models.BaseContent.objects.restricted(request).filter(*args, **kwargs)
    else:
        objs = lfc.models.BaseContent.objects.filter(*args, **kwargs)

    result = []
    for obj in objs:
        result.append(obj.get_content_object())

    return result

def get_portal(pk=1):
    """Returns the default portal.
    """
    # At the moment the default portal should always exist.
    cache_key = "portal-%s" % pk
    portal = cache.get(cache_key)
    if portal:
        return portal

    try:
        portal = lfc.models.Portal.objects.get(pk=pk)
    except lfc.models.Portal.DoesNotExist:
        portal = lfc.models.Portal.objects.filter()[0]

    cache.set(cache_key, portal)
    return portal

def traverse_object(request, path):
    """Returns the the object with the given path.
    """
    cache_key = "traverse-obj-%s" % path
    obj = cache.get(cache_key)
    if obj:
        return obj

    paths = path.split("/")
    language = translation.get_language()

    try:
        obj = lfc.utils.get_content_object(request, slug=paths[0],
            parent=None, language__in = ("0", language))
    except lfc.models.BaseContent.DoesNotExist:
        raise Http404

    for path in paths[1:]:
        try:
            obj = obj.children.restricted(request).get(slug=path, language__in = ("0", obj.language)).get_content_object()
        except lfc.models.BaseContent.DoesNotExist:
            raise Http404

    cache.set(cache_key, obj)
    return obj

def clear_cache():
    """Clears the complete cache.
    """
    # memcached
    try:
        cache._cache.flush_all()
    except AttributeError:
        pass
    else:
        return

    try:
        cache._cache.clear()
    except AttributeError:
        pass
    try:
        cache._expire_info.clear()
    except AttributeError:
        pass

def import_module(module):
    """Imports module with given dotted name.
    """
    try:
        module = sys.modules[module]
    except KeyError:
        __import__(module)
        module = sys.modules[module]
    return module

def getLOL(objects, objects_per_row=3):
    """Returns a list of list of given objects.
    """
    result = []
    row = []
    for i, object in enumerate(objects):
        row.append(object)
        if (i+1) % objects_per_row == 0:
            result.append(row)
            row = []

    if len(row) > 0:
        result.append(row)

    return result

def lfc_quote(string, encoding="utf-8"):
    """Encodes string to encoding before quoting.
    """
    return urllib.quote(string.encode(encoding))

# TODO: Not used at the moment - what to do?
def get_related_pages_by_tags(page, num=None):
    """Returns a dict with related products by tags.

    This is just a thin wrapper for the get_related method of the
    TaggedItem manager of the tagging product in order to provide caching.
    From the tagging product's doc string (mutatis mutantis):

    Returns a list of products which share tags with the product with passed id
    ordered by the number of shared tags in descending order.

    See there for more.
    """
    # Try to get it out of cache
    cache_key = "related-page-by-tags-%s" % page.id
    related_pages = cache.get(cache_key)
    if related_pages is not None:
        return {"related_pages" : related_pages}

    # Create related pages
    related_pages = TaggedItem.objects.get_related(page, Page, num)

    # Save related pages to cache
    cache.set(cache_key, related_pages)

    return {"related_pages" : related_pages}



# python imports
import sys
import traceback

# django imports
from django import template
from django.conf import settings
from django.core.cache import cache
from django.core.mail import EmailMessage
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import Http404
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils import translation
from django.utils.translation import ugettext_lazy as _

# lfc imports
import lfc.utils
from lfc.utils import traverse_object
from lfc.models import File
from lfc.models import BaseContent
from lfc.models import Portal

# tagging imports
from tagging.models import TaggedItem
from tagging.utils import get_tag

def portal(request, obj, template_name="lfc/portal.html"):
    """Displays the the portal.
    """
    return render_to_response(template_name, RequestContext(request, {
        "portal" : lfc.utils.get_portal()
    }))

def base_view(request, language=None, slug=None, obj=None):
    """Displays the object for given language and slug.
    """
    language = translation.get_language()
    # If the given language is the default language redirect to the url without
    # the language code http:/domain.de/de/hurz = http:/domain.de/hurz

    # Get the obj (passed my LFC Middleware)
    obj = request.META.get("lfc_context")

    if obj is None:
        raise Http404()

    if isinstance(obj, Portal):
        return portal(request, obj)

    # Redirect to standard object unless superuser is asking
    if obj.standard and request.user.is_superuser == False:
        url = obj.get_absolute_url()
        return HttpResponseRedirect(url)

    # Template
    template_cache_key = "template-%s-%s" % (obj.content_type, obj.id)
    obj_template = cache.get(template_cache_key)
    if obj_template is None:
        obj_template = obj.get_template()
        cache.set(template_cache_key, obj_template)

    # Children
    children_cache_key = "children-%s-%s" % (obj.content_type, obj.id)
    sub_objects = cache.get(children_cache_key)
    if sub_objects is None:
        # Get sub objects (as LOL if requested)
        if obj_template.children_columns == 0:

            sub_objects = obj.children.restricted(request).get_content_objects()
        else:
            sub_objects = lfc.utils.getLOL(obj.children.restricted(request).get_content_objects(), obj_template.children_columns)

        cache.set(children_cache_key, sub_objects)

    # Images
    images_cache_key = "images-%s-%s" % (obj.content_type, obj.id)
    cached_images = cache.get(images_cache_key)
    if cached_images:
        image     = cached_images["image"]
        images    = cached_images["images"]
        subimages = cached_images["subimages"]
    else:
        temp_images = list(obj.images.all())
        if temp_images:
            if obj_template.images_columns == 0:
                images = temp_images
                image = images[0]
                subimages = temp_images[1:]
            else:
                images = lfc.utils.getLOL(temp_images, obj_template.images_columns)
                subimages = lfc.utils.getLOL(temp_images[1:], obj_template.images_columns)
                image = images[0][0]
        else:
            image = None
            images = []
            subimages = []

        cache.set(images_cache_key, {
            "image" : image,
            "images" :  images,
            "subimages" : subimages
        })

    # Files
    files = obj.files.all()

    c = RequestContext(request, {
        "lfc_context" : obj,
        "images" : images,
        "image" : image,
        "subimages" : subimages,
        "files" : files,
        "sub_objects" : sub_objects,
        "portal" : lfc.utils.get_portal(),
    })

    # Render twice. This makes tags within text / short_text possible.
    result = render_to_string(obj_template.path, c)
    result = template.Template("{% load lfc_tags %} " + result).render(c)

    return HttpResponse(result)

def file(request, language=None, id=None):
    """Delivers files to the browser.
    """
    file = get_object_or_404(File, pk=id)
    response = HttpResponse(file.file, mimetype='application/binary')
    response['Content-Disposition'] = 'attachment; filename=%s' % file.title

    return response

def search_results(request, language, template_name="lfc/search_results.html"):
    """Displays the search result for passed language and query.
    """
    query = request.GET.get("q")
    language = translation.get_language()

    f = Q(exclude_from_search=False) & \
        (Q(language = language) | Q(language="0")) & \
        (Q(searchable_text__icontains=query))

    try:
        obj = BaseContent.objects.restricted(request).get(slug="search-results")
    except BaseContent.DoesNotExist:
        obj = None

    results = BaseContent.objects.restricted(request).filter(f)
    return render_to_response(template_name, RequestContext(request, {
        "lfc_context" : obj,
        "query" : query,
        "results" : results,
    }))

def set_language(request, language, id=None):
    """Sets the language to the given language

    parameters:

        - language
          the requested language

        - id:
          the id of the current displayed object
    """
    translation.activate(language)

    url = None
    if id:
        obj = BaseContent.objects.restricted(request).get(pk=id)

        # If the language of the current object same as the requested language we
        # just stay on the object.
        if obj.language == language:
            url = obj.get_absolute_url()

        # Coming from a object with neutral language, we stay on this object
        elif obj.language == "0":
            url = obj.get_absolute_url()

        # Coming from a canonical object, we try to get the translation for the
        # given language
        elif obj.is_canonical():
            t = obj.get_translation(request, language)
            if t:
                url = t.get_absolute_url()
            else:
                if language == settings.LANGUAGE_CODE:
                    url = "/"
                else:
                    url = "/" + language

        # Coming from a translation, we try to get the canonical and display
        # the given language
        else:
            canonical = obj.get_canonical(request)
            if canonical:
                url = canonical.get_absolute_url()
            else:
                if language == settings.LANGUAGE_CODE:
                    url = "/"
                else:
                    url = "/" + language
    else:
        portal = lfc.utils.get_portal()
        if language == settings.LANGUAGE_CODE:
            url = "/"
        else:
            url = "/" + language

    response = HttpResponseRedirect(url)

    if translation.check_for_language(language):
        if hasattr(request, 'session'):
            request.session['django_language'] = language
        else:
            response.set_cookie(settings.LANGUAGE_COOKIE_NAME, language)

    return response

def lfc_tagged_object_list(request, slug, tag, template_name="lfc/page_list.html"):
    """
    """
    if tag is None:
        raise AttributeError(_('tagged_object_list must be called with a tag.'))

    tag_instance = get_tag(tag)
    if tag_instance is None:
        raise Http404(_('No Tag found matching "%s".') % tag)

    obj = request.META.get("lfc_context")
    queryset = BaseContent.objects.restricted(request).filter(parent=obj)
    objs = TaggedItem.objects.get_by_model(queryset, tag_instance)

    return render_to_response(template_name, RequestContext(request, {
        "slug" : slug,
        "lfc_context" : obj,
        "objs" : objs,
        "tag" : tag,
    }));

def fiveohoh(request, template_name="500.html"):
    """Handler for 500 server errors. Mails the error to ADMINS.
    """
    exc_type, exc_info, tb = sys.exc_info()
    response = "%s\n" % exc_type.__name__
    response += "%s\n" % exc_info
    response += "TRACEBACK:\n"
    for tb in traceback.format_tb(tb):
        response += "%s\n" % tb

    response += "\nREQUEST:\n%s" % request

    try:
        from_email = settings.ADMINS[0][1]
        to_emails = [a[1] for a in settings.ADMINS]
    except IndexError:
        pass
    else:
        mail = EmailMessage(
            subject="Error LFC", body=response, from_email=from_email, to=to_emails)
        mail.send(fail_silently=True)

    return base_view(request, slug="500")
